package org.codehaus.plexus.classworlds.strategy;

/*
 * Copyright 2001-2006 Codehaus Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;

import org.codehaus.plexus.classworlds.realm.ClassRealm;

/**
 * A strategy is a class for defining how classes and resources are located
 * in classworlds.
 *
 * @uthor: Andrew Williams
 * @since: Nov 19, 2006
 */
public interface Strategy
{

    Class<?> loadClass( String name )
        throws ClassNotFoundException;

    URL getResource( String name );

    Enumeration<URL> getResources( String name )
        throws IOException;

    ClassRealm getRealm();

}
